{
               SCREEN ACCESSARY
               -------------------------------------
               Version      :   2.00 (OOP)
               Unit name    :   Box
               Code name    :   Mesmeric Accessary
               Last Created :   1994/12/14
               Enviroment   :   Color/Monochrome
               Function     :   drawing colorful boxes
               Code         :   KIM, JIN HONG [Mesmeric]
               Belong to    :   PENTAGRAM
               -------------------------------------
}

{$R-}

unit Box;

interface

uses
  Crt, PublVar, TextFont;

type
      PBox=object
        Top,Left,Bottom,Right,
        LBC,                  { Line Back Color }
        THK,                  { Thickness }

        WKI,                  { What kind of icon ? }
        IC,                   { Icon color        }
        TMC:byte;             { Top Message Color }

        TopMessage:string;
        Standing,
        ShadowOn,
        PaintIn   :boolean;

        constructor Init;          { initialize variables for box }
        destructor  Done; virtual; { dispose screen buffer        }

        procedure   SetXY2(T,L,B,R:byte); virtual; { set box coordinates }
        procedure   SetBoxColor(LC,BC:byte);       { box line/background color }
        procedure   Thickness(TH:byte);            { box line thickness }
        procedure   Draw; virtual;                 { draw box           }
        procedure   SetShadow(NF:boolean);         { setting shadow on/off }
        procedure   SetPaint(NF:boolean);

        procedure   PutTop(S:string;M:boolean);    { top of box message }
        procedure   SetTopColor(LC,BC:byte);
        procedure   SetIcon(K:byte);
        procedure   SetIconColor(LC,BC:byte);
      end;

      PMovingBox=object(PBox)
        SaveScreen :^ScrnCell;

        constructor Init;
        destructor  Done; virtual;
        procedure   ReMove;
        procedure   Move(T,L,B,R:byte); { set new coordinates }
      end;

      PZoom=object(PBox)
        DelayValue  : word;

        constructor Init;
        procedure   SetDelay(D:word);
        procedure   Draw; virtual;
      end;

procedure ExtendedFontEnable;
procedure ExtendedFontDisable;

implementation

const
  MaxFont        =14;
  FontModifyStart=188;
  BoxNewFont:array[1..MaxFont] of FontType=((255,1,1,1,1,1,1,249,249,1,1,1,1,1,1,255),
                                            (0,0,224,28,2,1,1,1,2,28,224,0,0,0,0,0),
                                            (0,0,224,28,2,193,225,193,2,28,224,0,0,0,0,0),
                                            (192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192),
                                            (1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1),
                                            (255,255,1,1,1,1,1,1,1,1,1,1,1,1,1,1),
                                            (1,1,1,1,1,1,1,1,1,1,1,1,1,1,255,255),
                                            (255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
                                            (0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255),
                                            (255,255,192,192,192,192,192,192,192,192,192,192,192,192,192,192),
                                            (192,192,192,192,192,192,192,192,192,192,192,192,192,192,255,255),
                                            (255,128,128,128,128,128,128,159,159,128,128,128,128,128,128,255),
                                            (0,0,7,56,64,128,128,128,64,56,7,0,0,0,0,0),
                                            (0,0,7,56,64,131,135,131,64,56,7,0,0,0,0,0));
type
  IconCell = array[1..2] of byte;

const
  MaxIcon = 3;
  Icons   :array[1..MaxIcon] of IconCell=((91,93),(199,188),(201,190));

var
     SaveOrgFonts  :array[1..MaxFont] of FontType;
     ContainNewFont:boolean;
     SaveExitProc  :pointer;

procedure ExtendedFontDisable;
var T:byte;
begin
  if ContainNewFont then begin
    for T:=1 to MaxFont do SetFont(pred(FontModifyStart)+T,SaveOrgFonts[T]);
    ContainNewFont:=Off;
    exitproc:=SaveExitProc
  end
end;

procedure ExtendedFontEnable;
var T:byte;
begin
  for T:=1 to MaxFont do begin
    GetFont(pred(FontModifyStart)+T,SaveOrgFonts[T]);
    SetFont(pred(FontModifyStart)+T,BoxNewFont[T])
  end;

  SaveExitProc  :=exitproc;
  exitproc      :=@ExtendedFontDisable;
  ContainNewFont:=On
end;

(* ------------------------------------------------------------------- Box *)
procedure PBox.SetBoxColor;
begin LBC:=LC+16*BC end;

constructor PBox.Init;
begin
  Top     :=1;
  Left    :=1;
  Bottom  :=25;
  Right   :=80;
  THK     :=1;

  TopMessage :='';
  WKI        :=0;
  Standing   :=On;
  ShadowOn   :=Off;
  PaintIn    :=On;

  SetBoxColor(15,1);
  SetTopColor(1,7);
  SetIconColor(15,0)
end;

destructor PBox.Done;
begin end;

procedure PBox.SetXY2;
begin
  Top    := T;
  Left   := L;
  Bottom := B;
  Right  := R
end;

procedure PBox.Thickness;
begin THK:=TH end;

procedure PrintMessage(TopMessage:string;Top,Left,Bottom,
                       Right,TMC,WKI:byte;Standing:boolean);
var T,L,P:byte;
begin
  L:=length(TopMessage);
  if WKI>0 then P:=2 else P:=0;

  if TopMessage<>'' then begin
    for T:=Left+P to Right do
      Scrn^[Top,T]:=CCW(#190,(TMC div 16)+16*(TMC div 16));

    if Standing then
      for T:=1 to L do
        Scrn^[Top,Left+(pred(Right-Left-L+P) div 2)+T]:=CCW(TopMessage[T],TMC)
    else
      for T:=1 to L do
        Scrn^[Top,pred(Left+T)+P]:=CCW(TopMessage[T],TMC)
  end
end;

procedure PBox.SetIcon;
begin WKI:=K end;

procedure PBox.SetShadow;
begin ShadowOn:=NF end;

procedure ShadowColor(var W:word);
var LC,BC:byte;
begin
  LC:=W div 256 mod 16;
  BC:=W div 256 div 16;
  if LC<9 then LC:=8 else dec(LC,8);
  if BC<9 then BC:=0 else dec(BC,8);
  W:=word(LC+BC*16) shl 8 + W mod 256
end;

procedure PBox.SetPaint;
begin PaintIn:=NF end;

procedure PBox.Draw;
var T,I,B2,R2:byte;
    CL:string[6];
begin
  B2:=Bottom;
  R2:=Right;
  if B2>25 then B2:=25;
  if R2>80 then R2:=80;
  if Left>80 then Left:=80;
  if Top >25 then Top:=25;

  { paint in box }
  if PaintIn then
    for T:=Top to B2 do
      for I:=Left to R2 do
        Scrn^[T,I]:=word(LBC) shl 8 + 32;

  if ContainNewFont then THK:=6;

  case THK of
    0:CL:='';
    1:CL:='ڿٳ';
    2:CL:='ɻȼ';

    3:begin
        for T:=succ(Left) to pred(Right) do begin
          Scrn^[Top,T]   :=CCW('',LBC);
          Scrn^[Bottom,T]:=CCW('',LBC)
        end;
        for T:=Top to Bottom do begin
          Scrn^[T,Left] :=CCW('',LBC);
          Scrn^[T,Right]:=CCW('',LBC)
        end
    end;

    4:begin
        for T:=Left to Right do begin
          Scrn^[Top,T]   :=CCW('',LBC);
          Scrn^[Bottom,T]:=CCW('',LBC)
        end;
        for T:=Top to Bottom do begin
          Scrn^[T,Left]       :=CCW('',LBC);
          Scrn^[T,succ(Left)] :=CCW('',LBC);
          Scrn^[T,Right]      :=CCW('',LBC);
          Scrn^[T,pred(Right)]:=CCW('',LBC)
        end
    end;

    5:begin
        for T:=Left to Right do begin
          Scrn^[Top,T]   :=CCW('',LBC);
          Scrn^[Bottom,T]:=CCW('',LBC)
        end;
        for T:=succ(Top) to pred(Bottom) do begin
          Scrn^[T,Left] :=CCW('',LBC);
          Scrn^[T,Right]:=CCW('',LBC)
        end
    end;

    6:begin
        for T:=succ(Left) to pred(Right) do
          Scrn^[Top,T]  :=CCW(#195,LBC);
        Scrn^[Top,Left] :=CCW(#197,LBC);
        Scrn^[Top,Right]:=CCW(#193,LBC);

        Scrn^[Bottom,Left] :=CCW(#198,LBC);
        Scrn^[Bottom,Right]:=CCW(#194,LBC);

        for T:=succ(Left) to pred(Right) do
          Scrn^[Bottom,T]:=CCW(#196,LBC);

        for T:=succ(Top) to pred(Bottom) do begin
          Scrn^[T,Left] :=CCW(#191,LBC);
          Scrn^[T,Right]:=CCW(#192,LBC)
        end
    end
  end;

  if THK in [0..2] then begin
    for T:=succ(Left) to pred(Right) do begin
      Scrn^[Top,T]   :=CCW(CL[6],LBC);
      Scrn^[Bottom,T]:=CCW(CL[6],LBC)
    end;

    for T:=succ(Top) to pred(Bottom) do begin
      Scrn^[T,Left] :=CCW(CL[5],LBC);
      Scrn^[T,Right]:=CCW(CL[5],LBC)
    end;

    Scrn^[Top,Left]    :=CCW(CL[1],LBC);
    Scrn^[Top,Right]   :=CCW(CL[2],LBC);
    Scrn^[Bottom,Left] :=CCW(CL[3],LBC);
    Scrn^[Bottom,Right]:=CCW(CL[4],LBC)
  end;

  if length(TopMessage) > 0 then begin
    PrintMessage(TopMessage,Top,Left,Bottom,Right,TMC,WKI,Standing);

    if WKI>0 then
      if ContainNewFont then begin
        Scrn^[Top,Left]      :=CCW(chr(Icons[WKI,1]),IC);
        Scrn^[Top,succ(Left)]:=CCW(chr(Icons[WKI,2]),IC)
      end else begin
            Scrn^[Top,Left]      :=CCW(chr(Icons[1,1]),IC);
            Scrn^[Top,succ(Left)]:=CCW(chr(Icons[1,2]),IC)
      end
  end;

  if ShadowOn then begin
    for T:=succ(Top) to succ(Bottom) do begin
      ShadowColor(Scrn^[T,succ(Right)]);
      ShadowColor(Scrn^[T,succ(succ(Right))])
    end;
    for T:=succ(succ(Left)) to Right do ShadowColor(Scrn^[succ(Bottom),T])
  end
end;

procedure PBox.SetIconColor;
begin IC:=LC+16*BC end;

procedure PBox.SetTopColor;
begin TMC:=LC+16*BC end;

procedure PBox.PutTop;
begin
  TopMessage:=S;
  Standing  :=M
end;
(* ------------------------------------------------------------------- Box *)

constructor PMovingBox.Init;
begin
  PBox.Init;
  new(SaveScreen);
  SaveScreen^:=Scrn^
end;

destructor PMovingBox.Done;
begin
  PBox.Done;
  dispose(SaveScreen)
end;

procedure PMovingBox.ReMove;
begin Scrn^:=SaveScreen^ end;

procedure PMovingBox.Move;
begin
  Top     := T;
  Left    := L;
  Bottom  := B;
  Right   := R;
  Scrn^   := SaveScreen^;
  Draw
end;

(* ------------------------------------------------------------------ Zoom *)
constructor PZoom.Init;
begin
  PBox.Init;
  DelayValue := 25
end;

procedure PZoom.SetDelay;
begin DelayValue:=D end;

procedure PZoom.Draw;
var SaveOrgXY2:array[1..4] of byte;
    WideX,WideY,Count:byte;
begin
  SaveOrgXY2[1]:= Top;
  SaveOrgXY2[2]:= Left;
  SaveOrgXY2[3]:= Bottom;
  SaveOrgXY2[4]:= Right;

  Count :=0;
  WideX :=(Right-Left) div 10;
  WideY :=(Bottom-Top) div 10;

  if WideX=0 then WideX:=(Right-Left) div 2;
  if WideY=0 then begin
    WideY:=(Bottom-Top) div 2;
    Count:=4
  end;

  Top  := (Bottom-Top) div 2+Top;
  Left := (Right-Left) div 2+Left;

  if Top  = 0  then Top  := 1;
  if Left = 0  then Left := 1;

  Bottom  := succ(Top);
  Right   := succ(Left);

  repeat
    delay(DelayValue);
    inc(Count);

    PBox.Draw;

    if Top   > SaveOrgXY2[1] then dec(Top,WideY);
    if Left  > SaveOrgXY2[2] then dec(Left,WideX);
    if Bottom< SaveOrgXY2[3] then inc(Bottom,WideY);
    if Right < SaveOrgXY2[4] then inc(Right,WideX);

    if Top   < SaveOrgXY2[1] then Top   := SaveOrgXY2[1];
    if Left  < SaveOrgXY2[2] then Left  := SaveOrgXY2[2];
    if Bottom> SaveOrgXY2[3] then Bottom:= SaveOrgXY2[3];
    if Right > SaveOrgXY2[4] then Right := SaveOrgXY2[4]
  until Count=5;

  delay(DelayValue);
  SetXY2(SaveOrgXY2[1],SaveOrgXY2[2],SaveOrgXY2[3],SaveOrgXY2[4]);
  PBox.Draw
end;
(* ------------------------------------------------------------------ Zoom *)

end.
