{
               SCREEN TOOLS - help programmer to control the screen
               ----------------------------------------------------
               Version      :   2.31
               Unit name    :   Screen
               Code name    :   Screen Driver
               Last Created :   1995/09/16
               Enviroment   :   Color Only
               Function     :   management of the screen
               Code         :   KIM, JIN HONG ( AROMANIX / Digital Art )
               ----------------------------------------------------
}

{$R-}

unit Screen;

interface
uses
  crt,dos,publvar;

const
  { Color Table }
  _CT:array[0..15] of byte=(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15);
  _DISPSEG:word=$B800;   { segment of display }

var
  _CRT_COLS:word absolute $0040:$004A;
  _CRT_ROWS:word absolute $0040:$0084;

  procedure Cls;
  procedure Cursor(T:boolean);
  procedure CursorHeight(S,E:byte);
  procedure BlinkAttrib(T:boolean);
  procedure Video(T:boolean);
  procedure Mode(M:word);
  procedure Sounds(H,D,AD:word);
  procedure WaitVirt;
  procedure Print(S:string;C,B:byte;M:boolean);
  procedure FPrint(S:string;X,Y,C:word);
  procedure ScrollUp;
  procedure SaveScreen(T,L,B,R:byte;var Buf:pointer);
  procedure RestoreScreen(X,Y:byte;var Buf:pointer;C:boolean);
  procedure PushScreen;
  procedure PopScreen;

  function IsColor:boolean;
  function IsEgaVga:boolean;
  function IsVga:boolean;
  function Num2Str(N:longint):string;
  function Str2Num(S:string):longint;
  function Lz(W:byte;zero:boolean):string;
  function Spc(C:char;T:byte):string;
  function UCase(S:string):string;

implementation

type
  Link=^Node;
  Node=record
    Data:pointer;
    Prev:Link
  end;

var
  Inctr:Link;

procedure PushScreen;
var T:Link;
begin
  new(T);
  SaveScreen(1,1,25,80,T^.Data);
  T^.Prev:=Inctr;
  Inctr:=T
end;

procedure PopScreen;
var T:Link;
begin
  T:=Inctr;
  RestoreScreen(1,1,T^.Data,true);
  Inctr:=T^.Prev;
  dispose(T)
end;

procedure RestoreScreen(X,Y:byte;var Buf:pointer;C:boolean);
var T,S,NY:byte;
    I:^byte;
begin
  I:=Buf;
  S:=I^;  { size of x }
  asm inc word ptr I end;   { increase pointer }
  NY:=I^;
  asm inc word ptr I end;   { increase pointer }

  for T:=1 to NY do begin
    move(I^,ptr(_DISPSEG,(Y+T-2)*160+(X-1)*2)^,S);
    inc(word(I),S)
  end;
  if C then freemem(Buf,NY*S+2)
end;

procedure SaveScreen(T,L,B,R:byte;var Buf:pointer);
var I:^byte;
    P,S:byte;
begin
  getmem(Buf,(R-L+1)*(B-T+1)*2+2);
  I:=Buf;

  S:=(R-L+1)*2;
  I^:=S;
  asm inc word ptr I end;   { increase pointer }
  I^:=B-T+1;
  asm inc word ptr I end;   { increase pointer }

  for P:=T to B do begin
    move(ptr(_DISPSEG,(P-1)*160+(L-1)*2)^,I^,S);
    inc(word(I),S)
  end
end;

procedure ScrollUp; assembler;
asm
        push    ds
        mov     es,_DISPSEG
        mov     ds,_DISPSEG
        mov     si,160
        mov     di,0
        mov     cx,1920
        cld
        rep     movsw
        pop     ds
end;

procedure FPrint(S:string;X,Y,C:word); assembler;
asm
        push    ds

        mov     ax,y
        dec     ax
        mov     bl,160
        mul     bl

        mov     di,x
        dec     di
        shl     di,1

        add     di,ax

        mov     es,_DISPSEG
        lds     si,S       { DS:SI points to S }

        mov     ch,0
        mov     cl,ds:[si]
        cmp     cl,0
        je      @esc

        inc     si
        mov     ah,byte ptr [C]
        cld
@cont:
        mov     al,ds:[si]
        mov     es:[di],ax
        add     di,2
        inc     si
        loop    @cont
@esc:
        pop     ds
end;

procedure WaitVirt; assembler;
asm
        mov  dx, 3daH
@cont:
        in   al, dx
        test al, 08H
        jz   @cont
end;

procedure Sounds;
begin
  sound(H);
  delay(D);
  nosound;
  delay(AD)
end;

procedure Print(S:string;C,B:byte;M:boolean);
begin
  textattr:=C+16*B;
  write(S);
  textattr:=7;
  if M then writeln
end;

procedure Cls; assembler;
asm
    mov     ax,1824
    mov     es,_DISPSEG
    mov     di,0
    mov     cx,2000
    cld
@cont:
    mov     es:[di],ax
    add     di,2
    loop    @cont
end;

function Str2Num(S:string):longint;
var C:word;
    N:longint;
begin
  val(S,N,C);
  Str2Num:=N
end;

function Num2Str(N:longint):string;
var S:string;
begin
  str(N,S);
  Num2Str:=S
end;

function Spc;
var I:byte;
    S:string;
begin
  S:='';
  for I:=1 to T do S:=S+C;
  Spc:=S
end;

function IsEgaVga:boolean; assembler;
asm
    MOV AH, 12h
    MOV BL, 10h  {yes, BL!}
    INT 10h
    MOV AL, 0
    CMP BH, 1
    JA @NoEgaVga
    CMP BL, 3
    JA @NoEgaVga
    INC AL
    @NoEgaVga:
END;

FUNCTION IsVga : Boolean; Assembler;
ASM
    MOV AH, 12h
    MOV AL, 00h
    MOV BL, 36h
    INT 10h
    MOV AH, 0
    CMP AL, 12h
    JNZ @NotVga
    INC AH
    @NotVga:
END;

procedure Mode; assembler;
asm
        mov     ax,M
        int     10h
end;

procedure Video; assembler;
asm
        mov     ah,12h        { BIOS function }
        mov     al,T          { 0 = on, 1 = off }
        mov     bl,36h        { Subfunction }
        int     10h           { Call BIOS }
end;

function UCase;
var T:byte;
begin
  for T:=1 to length(S) do S[T]:=upcase(S[T]);
  UCase:=S
end;

function IsColor;
begin
  if mem[$000:$449]=7 then IsColor:=false else IsColor:=true
end;

procedure CursorHeight(S,E:byte); assembler;
asm
        mov  ah,1
        mov  ch,S
        mov  cl,E
        int  10h
end;

procedure BlinkAttrib(T:boolean); assembler;
asm
        mov   ah,10h
        mov   al,3
        cmp   T,0
        jnz   @on
        mov   bl,0
        jmp   @con
@on:
        mov   bl,1
@con:
        int 10h
end;

procedure Cursor; assembler;
asm
        mov   ah,1h
        cmp   T,0
        jnz   @false
        mov   cx,3030h
        jmp   @cont
@false:
        mov   cx,0b0ch
@cont:
        int   10h
end;

function Lz;
var S:string;
begin
  str(W,S);
  if length(S)=1 then begin
     if zero then Lz:='0'+S else lz:=' '+s;
  end else Lz:=S
end;

begin
  Inctr:=nil;
end.